
getDerivatives := proc()

local n, idx;
local dXnFlow, dXdRdt, QQ;
local RIDs;

global dX, dS;

local dTKS, dTKG, TK;
local RVfill, RVempt;
local limFill, limEmpt;

local tmpX, tmpA, tmpB;


	dX 	:= Vector();
	dS 	:= Vector();
	Xexp 	:= Vector();
	QQ 	:= Vector();

	RIDs := map2(cat,R,reactionIDs);



### START ###
	printf("CALCULATING DERIVATIVES ... ");

### TEMPERATURES ###

	QQ(1) := Qcond 	= Acs*(1-eps) * nCells/length * lambda * (TKSin-2*TKS+TKSout);
	QQ(2) := Qconv 	= sAgeo * Vcat/nCells * alpha * (TKGout-TKS);
	QQ(3) := Qmt	= (TKGin-TKGout) * cpG * nFlow * MMair;

	dTKS := (Qcond + Qconv) / (rho*Vcat/nCells * cpS);
	dTKG := (Qmt - Qconv)  / (Vcat/nCells * eps * cpG * c0 * MMair);


### SURFACE SPECIES ###

	for n to numelems(surface) do

		idx := ListTools[Search](surface[n],species);
		dS(n) := MTM[transpose](Vmat[..,idx]).RIDs / cat(surface[1],'SC');
	
	end do;


### CONCENTRATION DERIVATIVE ###

	for n to numelems(xGases) do
	
		idx := ListTools[Search](xGases[n],species);

		dXdRdt  := - MTM[transpose](Vmat[..,idx]).RIDs;

		dXnFlow := nFlow * (cat(xGases[n],"in") - cat(xGases[n],"out")) * nCells/(Vcat*eps);
		dX(n) := 1/c0 *  (dXnFlow + dXdRdt);

	end do;


### SIMPLIFICATIONS ###

	RR := rr;
	for n to numelems(surface) do
		RR := eval(RR, map2(cat, c, surface[1]) = 1-MTM[sum](map2(cat, c, xSurf[1])));
	end do;

	RR := eval(RR, convert(map2(cat, 'c', gases) =~ gases *~ c0,set));
	RR := eval(RR, convert(staticGases,set));

	if (stateApprox = 2) then
		RR := eval(RR, convert(xGases =~ (map(cat,xGases,"in")/2 +~ map(cat,xGases,"out")/2),set));
	elif (stateApprox = 3) then
		RR := eval(RR, convert(xGases =~ map(cat,xGases,"out"),set));
	else
		RR := eval(RR, convert(xGases =~ map(cat,xGases,"in"),set));
	end if;

	RR := eval(RR, convert(map2(cat, 'c', xSurf) =~ xSurf,set));					# cV_NH3 = V_NH3;
#	RR := eval(RR, convert(map2(cat, 'c', xSurf) =~ xSurf *~ cat(op(surface),'SC'),set));		# cV_NH3 = V_SC * V_NH3;

	RR := RIDs =~ eval(RR, solve(convert(dX, set), convert(map(cat, xGases, out), set)));
	RR := solve(convert(RR, set), convert(RIDs, set));

#	RR := allvalues(RR)[4];		# SCR only
	RR := convert(RR, list);


### REACTION LIMITS ###

	RLim := Vector();

	# Limit to inlet
	tmpB := eval(dX,convert(map(cat, xGases, out) =~ 0, set));
	tmpA := Matrix(numelems(dX),numelems(RIDs));
	for n to numelems(RIDs) do
		tmpA[..,n] := map(solve, convert(tmpB,Vector), RIDs[n]);
	end;
	
	for n to numelems(RIDs) do
		RLim(numelems(RLim)+1) := RIDs[n] = min(RIDs[n], min(tmpA[..,n]));
	end;

	# Limit to surface species
	RVfill := (1-xSurf[1]) * cat(surface[1],SC) / dt;
	RVempt := xSurf[1] * cat(surface[1],SC) / dt;

	idx := ListTools[Search](xSurf[1],species);

	for n to numelems(RIDs) do
#			limFill := solve(RVfill = - Vmat(1..n, idx).convert(map2(cat, R, [`$`(1..n)]), Vector), RIDs[n]);
			limFill := solve(RVfill = - Vmat(1..n, idx).convert(map(RIDs, [`$`(1..n)]), Vector), RIDs[n]);
			limEmpt := solve(RVempt =   Vmat(1..n, idx).convert(map(RIDs, [`$`(1..n)]), Vector), RIDs[n]);
		if (Vmat(n,idx) < 0) then
				RLim(numelems(RLim)+1) := RIDs[n] = min(RIDs[n],limFill);
			if evalb(reverse(n) = 1) then
				RLim(numelems(RLim)+1) := RIDs[n] = max(RIDs[n],limEmpt);
			end if;
		else
				RLim(numelems(RLim)+1) := RIDs[n] = min(RIDs[n],limEmpt);
			if evalb(reverse(n) = 1) then
				RLim(numelems(RLim)+1) := RIDs[n] = max(RIDs[n],limFill);
			end if;
		end if;

	end;

### TEMPERATURE ###


	TK := [0 = dTKG, TKSnew = TKS+dt*dTKS];
	TK := eval(TK,convert(QQ,set));
	TK := eval(TK, lambda=0);

	Texp := convert(solve(TK, {TKGout, TKSnew}), list);


### EXPLICIT ###

#	Sexp := V_NH3new = V_NH3 + dt * dS[1];
	Sexp := map(cat, xSurf, new) =~ xSurf +~ dt *~ dS;
	Xexp := solve(convert(dX, list), convert(map(cat, xGases, out), list))[1];


	printf("DONE\n\n");
### END ###



end proc;




