
parseXML := proc(xmlFile)

	local XMLHandle;
	local reac, Re, Pr;
	local reactions;
	local id;
	local static;
	local n, i, idx;
	local others;
	local property;

	local tmpRe, tmpPr, tmpR, tmpA;

	reactions := [];
	reactionIDs := [];
	reverse := Vector();

	species := [];
	surface := [];
	xSurf := [];

	gases :=[];
	xGases := [];
	staticGases := Vector();
	

### START ###
	printf("PARSING REACTIONS ... \n");

	XMLHandle := XMLTools[ParseFile](xmlFile);
	reac := XMLTools[GetChildByName](XMLTools[GetChildByName]( XMLHandle, "reactions")[1],"reaction");

	for n to nops(reac) do

		id := parse(XMLTools[AttributeValue](reac[n],"id"));

		if parse(XMLTools[AttributeValue](reac[n],"active")) then

### REVERSE ###
			if parse(XMLTools[AttributeValue](reac[n],"reverse")) then
				reverse(n):=1;
				tmpR := "<->";
			else
				reverse(n):=0;
				tmpR := "->";
			end if;


### REACTIONS ###	
			Re := StringTools[Split](XMLTools[TextText](op(3, XMLTools[GetChildByName](reac[n], "reactant")[1])[1])," +");
			Pr := StringTools[Split](XMLTools[TextText](op(3, XMLTools[GetChildByName](reac[n], "product")[1])[1])," +");

			Re := remove(type,Re,"");
			Pr := remove(type,Pr,"");

			tmpRe := Re[1];
			for i from 2 to nops(Re) do
				if StringTools[IsAlpha](Re[i-1][1]) 	then tmpRe := cat(tmpRe, "+");	end if;
				if StringTools[IsDigit](Re[i-1][1]) 	then tmpRe := cat(tmpRe, "*");	end if;
				tmpRe := cat(tmpRe, Re[i]);				
			end do;

			tmpPr := Pr[1];
			for i from 2 to nops(Pr) do
				if StringTools[IsAlpha](Pr[i-1][1]) 	then tmpPr := cat(tmpPr, "+");	end if;
				if StringTools[IsDigit](Pr[i-1][1]) 	then tmpPr := cat(tmpPr, "*");	end if;
				tmpPr := cat(tmpPr, Pr[i]);
			end do;

			reactions   := Vector([reactions,parse(op(tmpRe)) - parse(op(tmpPr))]);
			reactionIDs := Vector([reactionIDs, id]);


### SPECIES ###
			for i to nops(Re) do
				if StringTools[IsAlpha](Re[i][1]) then
					species := [species, parse(Re[i])];
				end if;
			end do;

			for i to nops(Pr) do
				if StringTools[IsAlpha](Pr[i][1]) then
					species := [species, parse(Pr[i])];
				end if;
			end do;


			printf("   Reaction (%s) \t parsed successfully: %s %s %s\n", cat(id), tmpRe, tmpR, tmpPr);
		else

			printf("   Reaction (%s) \t disabled.\n", cat(id));

		end if;

	end do;

	species := ListTools[Flatten](species);
	species := remove(type,species,"");
	species := sort(ListTools[MakeUnique](species));

### OPTIONS ###

	stateApprox := parse(XMLTools[AttributeValue](XMLHandle, "stateApprox"));


### V MATRIX ###

	tmpA := Vector[column](nops(species));

	for n to nops(species) do
		tmpA[n] := Vector[row](nops(species), shape = unit[n]); 
	end do;

	Vmat := Matrix(ArrayTools[Size](reactions)[1], nops(species));

	for n to ArrayTools[Size](reactions)[1] do
		Vmat[n,..] := eval(reactions[n], convert(species =~ tmpA, list));
	end do;


### O MATRIX ###

	Omat := map(signum,Vmat);
#	Omat(3,7) := 2;
#	Omat(1,5) := -W1;
#	Omat := Vmat;


### STATIC GASES ###

	static := XMLTools[GetChildByName](XMLTools[GetChildByName]( XMLHandle, "staticGases")[1],"species");

	for n to numelems(static) do
		tmpA := parse(XMLTools[AttributeValue](static[n],"name"));
		if parse(XMLTools[AttributeValue](static[n],"const")) = true then
			staticGases(n) := tmpA = parse(XMLTools[TextText](op(3, static[n])[1]));
		else
			staticGases(n) := tmpA = tmpA;
		end if;
	end do;


### SURFACES & GASES ###

	gases := remove(StringTools[HasPunctuation], species);

	others := select(StringTools[HasPunctuation], species);

	for n to numelems(others) do
		tmpA := StringTools[Split](others[n],'_');
		if length(tmpA[2]) = 0 then
			surface := [op(surface), others[n]];
		end if;
	end do;


### SURFACE SPECIES ###

	xSurf := Vector();

	others := remove(member,others,surface);

	for n to numelems(others) do
		tmpA := StringTools[Split](others[n],'_');
		idx := ListTools[Search](tmpA[1],map2(StringTools[Select], StringTools[IsAlpha], surface));
		xSurf(idx) := others[n];
	end do;


### STATES ###

	xGases := remove(member, gases, convert(map(lhs, staticGases), list));

### GEOMETRY ###
	geo := Vector();
	property := XMLTools[GetChildByName](XMLTools[GetChildByName]( XMLHandle, "properties")[1],"constant");

	for n to numelems(property) do
		tmpA := parse(XMLTools[AttributeValue](property[n],"name"));
		if parse(XMLTools[AttributeValue](property[n],"active")) = true then
			geo(n) := tmpA = parse(XMLTools[TextText](op(3, property[n])[1]));
		end if;
	end do;

	printf("DONE\n\n");
### END ###

end proc;




