#include "mex.h"

#include "checkArrays.h"
#include "autocode.h"
#include "idx.h"
#include "io.h"

// u, p, nCells, Vcat, dtout, dt

void mexFunction( int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[] )
{
	double *t_out;
	const real_T *u, *p;
	double mFlow, tkg_us;
	double *exh_us;
	double t0, t;

	int i,m;

	if (nrhs == 0)
	{
		mexPrintf("\n");
		mexPrintf("model name:\t\t%s\n", MODELNAME);
		mexPrintf("model version:\t%s\n", MODELVERSION);
		mexPrintf("camgen version:\t%s\n", CAMGENVERSION);
		mexPrintf("date of build:\t%s\n", BUILDDATE);
		mexPrintf("\n");
		mexPrintf("parameters:\n%s", PARAMETERS);
		mexPrintf("\n");
	}
	else
	{

		nLeft = nlhs;

		checkArrays(nlhs, plhs, nrhs, prhs);
  
		/* Assign pointers to all inputs and outputs */
		u		= mxGetPr(prhs[0]);
		p		= mxGetPr(prhs[1]);

		t_out	= mxGetPr(plhs[0]);
		exh_ds	= mxGetPr(plhs[1]);
		tkg_ds	= mxGetPr(plhs[2]);
		if (nlhs > 3) {	osl_ds		= mxGetPr(plhs[3]);	}
		if (nlhs > 4) {	tks_ds		= mxGetPr(plhs[4]);	}
		if (nlhs > 5) {	exh_prof	= mxGetPr(plhs[5]);	}
		
		/* Allocate pointers IO */
		exh_us	= (double*)malloc(NGAS*sizeof(double));	
	

		t0		= u[0];		// initialize time to first input time
		mFlow	= u[1];
		tkg_us	= u[2];
		for (i=0; i<NGAS; i++)	{	exh_us[i] = max(0,u[i+NGAS+1]);		}

		camgen_init(tkg_us, exh_us);

		m = 0;

		/* Simulate */
		for (k=0; k<nSteps+1; k++)
		{
			t = t0 + k*dtout;
			t_out[k] = t;

			camgen_output();

			if ((t >= u[(m+1)*(NGAS+3)]) ) //&& (k<nSteps))
			{
				m++;
				mFlow  = u[1+m*(NGAS+3)];
				tkg_us = u[2+m*(NGAS+3)];
				for (i=0; i<NGAS; i++)	{	exh_us[i] = max(0,u[(i+3)+m*(NGAS+3)]);		}
			}		

			for (i=0; i < iSteps; i++)
			{
				camgen_update(p, mFlow, tkg_us, exh_us);
			};

		}

		free(exh_us);
		camgen_close();	
	}
 
}
