
#include "string.h"
#include "mex.h"
#include "idx.h"
#include "io.h"

int checkArrays(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
	mwSize mrows, ncols;

	double t0, t1, tend;

	/* Check for proper number of arguments. */
	if(nrhs < 4)
	{			
		mexErrMsgTxt("at least 4 input arguments required [data, para, nCells, Vcat, {dtout, dt}].");
	}
	
	if ((nlhs < 2) || (nlhs > 6))
	{
		mexErrMsgTxt("2-6 output arguments required.");
	}

	/* data */
	mrows = mxGetM(prhs[0]);
	ncols = mxGetN(prhs[0]);
	/*if( !mxIsDouble(prhs[1]) || mxIsComplex(prhs[1]) || !(mrows==NINPUTS+1) )*/
	if( !(mrows==NGAS+3) )
	{
		mexErrMsgTxt(" [data] needs to be of size [nInputs,nSteps]");
	}
	t0 = mxGetPr(prhs[0])[0];
	t1 = mxGetPr(prhs[0])[(ncols-2)*mrows];
	tend = mxGetPr(prhs[0])[(ncols-1)*mrows];

	/* para */
	mrows = mxGetM(prhs[1]);
	ncols = mxGetN(prhs[1]);
	if( !mxIsDouble(prhs[1]) || mxIsComplex(prhs[1]) || !(mrows*ncols==NPARAMS) )
	{
		//char *errString;
		 //(errString, "[para] needs to be of size [%d]",NPARAMS);
		//errString= strcat("[para] needs to be of size [", "]");
		mexErrMsgTxt("[para] needs to be of size [NPARAMS]");
	}

	nCells	= mxGetPr(prhs[2])[0];
	if(nCells < 1)
	{	
		mexErrMsgTxt("[nCells] needs to be a positive integer");
	}

	Vcat	= mxGetPr(prhs[3])[0];

	if(nrhs > 4)
	{
		dt = mxGetPr(prhs[4])[0];
	}
	else
	{
		dt = 1e-3;
	}

	if(nrhs > 5)
	{
		dtout = mxGetPr(prhs[5])[0];
	}
	else
	{
		int tmp = 1e6*(tend-t1);
		dtout = tmp / 1e6;
	}

	/* output */
	nSteps = (tend-t0) / dtout;		// calculate the number of output steps
	iSteps = dtout/dt;				// calculate the number of intermediate steps
	if(iSteps < 1)
	{
		iSteps = 1;
		mexWarnMsgTxt("dt has been adjusted to dtout!");
	}
	dt = dtout/iSteps;				// make dt a multiple of dtout
	

	plhs[0] = mxCreateDoubleMatrix(1,nSteps+1, mxREAL);
	plhs[1] = mxCreateDoubleMatrix(NGAS,nSteps+1, mxREAL);
	plhs[2] = mxCreateDoubleMatrix(1,nSteps+1, mxREAL);
	if (nlhs > 3) {	plhs[3] = mxCreateDoubleMatrix(nCells, nSteps+1, mxREAL);				}
	if (nlhs > 4) {	plhs[4] = mxCreateDoubleMatrix(nCells, nSteps+1, mxREAL);				}
	if (nlhs > 5) {	plhs[5] = mxCreateDoubleMatrix((nCells+1)*NGAS, nSteps+1, mxREAL);			}

}
